#[macro_export]
macro_rules! c_library
{
   {
      $(statics:
        {
           $($sname: ident: $stype: ty);+;
        }
      )?

      $(functions:
        {
           $($fname: ident($($farg: ident: $fargt: ty),*) -> $fret:ty);+;
        }
      )?

      $(varargs:
        {
           $($vname: ident($($vargs: ident: $vargst: ty),+) -> $vret:ty);+;
        }
      )?
   } =>
   {
      extern "C"
      {
         $($(pub static $sname: $stype;)+)?

         $($(pub fn $fname($($farg: $fargt),*) -> $fret;)+)?

         $($(pub fn $vname($(vargs: $vargst),+ , ...) -> $vret;)+)?
      }
   };
}

#[macro_export]
macro_rules! external_library
{
   {
      $name: ident: $link: expr =>
      {
         $(statics:
           {
              $($sname: ident: $stype: ty);+;
           }
         )?

         $(functions:
           {
              $($fname: ident($($farg: ident: $fargt: ty),*) -> $fret:ty);+;
           }
         )?

         $(varargs:
           {
              $($vname: ident($($vargs: ident: $vargst: ty),+) -> $vret:ty);+;
           }
         )?
      }
   } =>
   {
      #[link(name=$link)]
      extern "C"
      {
         $($(pub static $sname: $stype;)+)?

         $($(pub fn $fname($($farg: $fargt),*) -> $fret;)+)?

         $($(pub fn $vname($(vargs: $vargst),+ , ...) -> $vret;)+)?
      }
   };
}
